#include <sys/types.h>
#include <sys/shm.h>
#include <strings.h>
#include <errno.h>
#include <stdio.h>
#include <sys/wait.h>

#ifdef DEBUG
#define dprintf printf
#else
#define dprintf if (0) printf
#endif

int
main (void)
{
  key_t mykey;
  int shmid;
  char *testch = NULL;

  mykey = IPC_PRIVATE;

  dprintf ("creating private key... ");
  shmid = shmget (mykey, 65536, 0777);
  if (shmid != -1)
    {
      /* fork and then map from the forked process */
      int pid;
      int n;
      int status;
      dprintf ("id %x\n", shmid);
      if ((pid = fork ()) == 0)
	{
	  dprintf ("forked...\n");
	  dprintf ("attaching... ");
	  testch = (char *) shmat (shmid, NULL, 0);
	  if (testch == (char *) -1)
	    {
	      dprintf ("error %d, %s\n", errno, strerror (errno));
	      /* todo: delete the shm area */
	      exit (1);
	    }
	  dprintf ("mapped to %x\n", testch);
	  dprintf ("writing \'hello\' ...\n");
	  snprintf (testch, 6, "hello");
	  dprintf ("reading ... ");
	  dprintf ("'%s'...", testch);
	  if (strncmp (testch, "hello", 6))
	    {
	      dprintf ("failed, string mismatch\n");
	      exit (1);
	    }
	  else
	    dprintf ("ok\n");
	  exit (0);
	}
      if ((n = waitpid (pid, &status, 0)) != pid)
	{
	  dprintf ("wait pid failed, pid %d, n %d, errno %d\n", pid, n, errno);
	  exit (1);
	}
      else if (WIFEXITED (status))
	{
	  /* normal termination - pass child exist code onto our parent */
	  exit (WEXITSTATUS (status));
	}
	return 1;
    }
  else
    {
      dprintf ("error %d, %s\n", errno, strerror (errno));
      return 1;
    }

  return 0;
}
