#include <sys/types.h>
#include <sys/shm.h>
#include <strings.h>
#include <errno.h>
#include <stdio.h>

#ifdef DEBUG
#define dprintf printf
#else
#define dprintf if (0) printf
#endif


key_t ftok_check (char *name, int index)
{
  key_t temp;

  dprintf ("ftok (\'%s\',%d)...", name, index);
  temp = ftok (name, index);
  if (temp == (key_t) - 1)
    {
      dprintf ("error %d, %s\n", errno, strerror (errno));
      exit (1);
    }
  dprintf ("key value %0qx\n", temp);
  return temp;
}

int
main (void)
{
  key_t mykey;
  int shmid, shmid1, shmid2;

  mykey = ftok_check ("shmget.o", 1);
  dprintf ("creating shmget.o:1..");

  shmid1 = shmget (mykey, 65536, 0777 | IPC_CREAT);
  if (shmid1 != -1)
  {dprintf ("shm id %x\n", shmid1);}
  else
    {
      dprintf ("failed\nerror %d, %s\n", errno, strerror (errno));
      return 1;
    }

  dprintf ("creating second shmget.o:1 (check for same shmid)..");

  shmid2 = shmget (mykey, 65536, 0777 | IPC_CREAT);
  if (shmid2 == shmid1)
  {dprintf ("ok\n");}
  else if (shmid2 != -1)
    {
      dprintf ("failed\nWrong shm id returned: %x\n", shmid2);
      return 1;
    }
  else
    {
      dprintf ("error %d, %s\n", errno, strerror (errno));
      return 1;
    }

  mykey = ftok_check ("shmget.o", 20);
  dprintf ("creating exclusive shmget.o:2\n");

  shmid = shmget (mykey, 65536, 0777 | IPC_CREAT | IPC_EXCL);
  if (shmid != -1)
  {dprintf ("shm id %x\n", shmid);}
  else
    {
      dprintf ("error %d, %s\n", errno, strerror (errno));
      return 1;
    }

  dprintf ("creating exclusive shmget.o:2 (should fail)\n");
  shmid1 = shmget (mykey, 65536, 0777 | IPC_CREAT | IPC_EXCL);
  if (shmid1 != -1 || (shmid1 == -1 && errno != EEXIST))
    {
      dprintf ("unexpected pass: shm id %x\n", shmid1);
      return 1;
    }
  else
  {dprintf ("error %d, %s\n", errno, strerror (errno));}

  dprintf
    ("creating non-exclusive shmget.o:2 (should be the same as the prior id)\n");
  shmid2 = shmget (mykey, 65536, 0777 | IPC_CREAT);
  if (shmid == shmid2)
  {dprintf ("shm id %x\n", shmid2);}
  else
    {
      dprintf ("error %d, %s\n", errno, strerror (errno));
      return 1;
    }
  {
  FILE *f;
  if (!(f = fopen ("shmid.txt","wt")))
    {
      dprintf ("Could not open shmid.txt");
      return 1;
    }
  fprintf (f, "%0x\n",shmid);
  fclose (f);
  }
  return 0;
}
